/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: banner.c                                       *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will create a banner page based on      *
*         information in the users data structure.  It will    *
*         write the banner info	to a temporary file and then   *
*         call the process_file routine	to send the file to    *
*         the server.                                          *
*                                                              *
*       Input:                                                 *
*         Information in the following fields of the udp_data  *
*         structure will be used to form the banner page.      *
*                                                              *
*             usernam -- name of user submitting job           *
*             fname   -- name of file to be printed            *
*             host    -- server name                           *
*             svcnm   -- service name                          *
*                                                              *
*         In addition, the date, time and the current host     *
*         name will be included as part of the banner.         *
*                                                              *
*       Output:                                                *
*         The banner page will be sent to the server.  The     *
*         temporary files used to create the banner will be    *
*         deleted.                                             *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL 04-nov-93  Add a formfeed after the banner page.  This
                  was broken when the "formfeed" option was
                  added to the code.

   RKL 08-feb-93  Add a bit of casting to allow things to
                  compile cleanly.

   RKL 14-aug-92  Add pointer for character time in fprintf
                  rather than just putting function into
		  argument list to avoid prototype problems.

   RKL 31-jul-92  Replace bcopy with rtel_copy.  Amazing how
                  many support problems such a minor thing
		  can generate.

   RKL 05-mar-91  Replace strncpy with bcopy.

   RKL 29-mar-91  Remove form feed before banner page since
                  it is really not needed and wastes a sheet
		  of paper.

   RKL 09-jan-91  Changed RTAP to RTEL

   KCS 07-may-91  Added message field to errs() call.
 */


#include <udpinc.h>

int createbanner(users)
udp_data *users;
{
    udp_data banusr;
    FILE *ban;

    char hstnm[MAXLEN], *time_ptr;
    time_t clockval;

    rtel_copy(users, &banusr, sizeof(udp_data));
    banusr.flags = OPT_FORM;

    sprintf(banusr.fname, "/tmp/banner_%d", banusr.uniq_id);

    if ((ban = fopen(banusr.fname, "w")) == NULL) {
	errs(banusr, RTEL_NOBANNER, "banner:nobanner");
	errs(banusr, errno, "banner:nobanner");
	return PROC_ERR;
    }
    
    fprintf(ban, "\n\n\n\n\n\r");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL ");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL\r\n");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL ");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL\r\n");
    fprintf(ban, "\n\n\n\n\n\n\n");
    
    clockval = time(0);
    time_ptr = (char *) ctime(&clockval);
    gethostname(hstnm, MAXLEN);
    
    fprintf(ban, "   User       : %s\r\n", banusr.usernam);
    fprintf(ban, "   Date       : %s\r\n", time_ptr);
    fprintf(ban, "   Job Name   : %s\r\n", users->fname);
    fprintf(ban, "   Host       : %s\r\n", hstnm);
    if(strlen(banusr.svcnm)){
	fprintf(ban, "   Service    : %s\r\n", banusr.svcnm);
    }
    else{
	fprintf(ban, "   Port Num   : %s\r\n", banusr.hport);
    }
    fprintf(ban, "   Server     : %s\r\n", banusr.host);
    fprintf(ban, "   Version    : %s\r\n", VERSION);
    fprintf(ban, "\n\n\n\n\n\n\n");
    
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL ");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL\r\n");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL ");
    fprintf(ban, "RTEL RTEL RTEL RTEL RTEL RTEL RTEL RTEL\r\n");
    
    /* At this point, we have to flush the data out and seek back to the
       beginning of the file.  We really shouldn't have to do the flush,
       but if we don't it doesn't work... */
    
    fflush(ban);
    fclose(ban);
    
    if ((banusr.fdp = fopen(banusr.fname, "r")) != NULL) {
	process_file(&banusr);
	fclose (banusr.fdp);
    }
    unlink(banusr.fname);
    
    return OK;
}
