/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: becm_damon                                     *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine perform the steps necessary to become   *
*         an independent daemon process.  Note that since      *
*         these steps are different for SYSV verses BSD Unix,  *
*         this routine is ifdefed into two main parts.         *
*                                                              *
*       Input:                                                 *
*         No input is required for this routine.               *
*                                                              *
*       Output:                                                *
*         When this routine returns, the current process will  *
*         be running as an independent daemon process with no  *
*         controlling terminal and as a child of the root      *
*         process.                                             *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  05-oct-94  Added a sleep before the use of setpgrp
                   as some systems (aix) have problems when
                   the process group leader that we were
                   forked from is in the process of dying
                   and the system is in an inconsistant state.

   RKL  19-jun-92  Bit premature in use of setsid, as not
                   all systems support it yet.

   RKL  16-jun-92  Cleaned up and use setsid for SYSV type
                   systems.

   RKL  09-jan-91  Changed RTAP to RTEL.

   KCS  07-may-91  Added message field to errs() call.
   
 */

#include <udpinc.h>

become_daemon(users)
udp_data *users;
{
    int forkstat, fd;
  
    /* Start forking off children in order to make this process
       completely independent. */
  
    if((forkstat = fork()) < OK){
	errs(users, errno, "becm_damon:forkerr");
	return(PROC_ERR);
    }
    else if(forkstat){
	exit (0);
    }
  
    /* process child: in sysV we need to spawn again after doing some setup */
  
#ifdef BSD
    sleep(2);
    if(setpgrp(0, getpid()) < OK){
	errs(users, errno, "becm_damon:setpgrp");
	return(PROC_ERR);
    }
    if((fd = open("dev/tty", O_RDWR)) >= 0){
	ioctl (fd, TIOCNOTTY, (char*) NULL);
	close (fd);
    }
#else
    sleep(2);
    if (setpgrp() < OK){
	errs(users, errno, "becm_damon:setpgrp");
	return(PROC_ERR);
    }
    
    /* kick off second child, which will be the dissociated one */
    if((forkstat = fork()) < OK){
	errs(users, errno, "becm_damon:forkerr");
	return(PROC_ERR);
    }
    else if(forkstat){
	exit(0);
    }
#endif
    
    /* The daemon process now needs to be told to ignore a couple
       of signals and also needs to close all open file descriptors.
       In addition, change our default directory to root so we don't
       hang any file system changes and set our protection mask
       down so we don't create any unusual protection files. */
    
    signal_stuff();
    
    for (fd=0; fd < 10; fd++) close (fd);
    chdir ("/");
    umask (0);
    return OK;
}
