/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: conn_n_snd.c                                   *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will handle creating a connection to    *
*         the server and will then call the appropriate        *
*         routines to actually transfer the user file.         *
*                                                              *
*         If the service we are attempting to connect to is    *
*         currently busy, this routine is the one that sleeps  *
*         until the UDP daemon wakes us up.                    *
*                                                              *
*       Input:                                                 *
*         Information in the following fields of the udp_data  *
*         structure will be used to determine what to print.   *
*                                                              *
*            copies  -- Number of copies to print              *
*            fd      -- File descriptor of data to print       *
*            host    -- server name                            *
*            svcnm   -- service name                           *
*                                                              *
*         In addition, if a banner page is to be printed, the  *
*         following information is also used:                  *
*                                                              *
*            usernam -- name of user submitting job            *
*            fname   -- name of file to be printed             *
*                                                              *
*       Output:                                                *
*         Either the file will be successfully transfered to   *
*         the server or an error status will be returned       *
*         specifying what went wrong.                          *
*                                                              *
*         Messages (both informational and fatal) can also be  *
*         written to the spooler log file.                     *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  01-nov-93  Removed test_conn as it was never used.

   RKL  22-jan-91  Changed checking for queued status from connect
                   service call.

   KCS  29-oct-91  Changed the close to an fclose so that the stream is 
                   flushed before closing to be sure the data gets out of 
		   the stream.

		   If the telnet option was specified, add a timemark
		   to guarantee that data is sent and server output
		   buffer is cleared.

   RKL  23-feb-91  Only delete queue file when the connection
                   attempt finally completes.

   RKL  09-jan-91  Changed RTAP to RTEL.

 */

#include <udpinc.h>

char SERVER[MAXLEN];
char FILENAME[MAXLEN];

int conn_and_send(users)
udp_data *users;
{
    int rval;
    
    /* Save some info in global variables to be used by error routines
       and the signaling stuff. */
    
    strcpy(FILENAME, users->fname);
    strcpy(SERVER, users->host);

    users->flags |= ALLOW_QUE;
    
    while(TRUE){
	rval = connect_service(users);
	if(rval == OK) break;
	if(rval < OK) return(rval);
	waiting(users);
    }
    
    /* To accomodate users who do not want to print a banner page, the code
       to generate the banner is only called if banner is specified in the
       server_hosts configuration file.  However, even if the banner cannot
       be created, it is not a fatal error and the job will be printed.  */
    
    if (users->flags & OPT_BANNER) {
	createbanner(users);
    }
    
    if (users->copies <= 0) users->copies = 1;
    while (users->copies--) {
	fseek(users->fdp, 0, 0);
	if (rval = process_file(users) < OK) {
	    break;
	}
    }
    close (users->tcp_fd);
    return rval;
}
