/***************************************************************
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: do_stuff.c                                     *
*       Created: Thurday, Jan 16 02:59:22 1992                 *
*       Abstract:                                              *
*         This module contains all the do_xxxx routines called *
*         directly by the rteld process.                       *
*                                                              *
*       Input:                                                 *
*         As specified by each individual routine.             *
*                                                              *
*       Output:                                                *
*         As specified by each individual routine.             *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  13-may-94  Remove all references to kill_file stuff
                   and replace it with scanning the system
		   looking for rtel jobs.  This includes
		   completely removing the do_cleanup and
		   do_review routines.

   RKL  19-nov-92  Changed name server to trmsvr to fix a
                   conflict with some system include files.

   RKL  20-may-92  Allow none to be specified as an option
                   for rteld in the server_hosts file.

   RKL  20-may-92  Make "service not offered on server" an
                   informational message.  If fatal, cannot do
		   virtual translation through rteld.

   RKL  16-jan-92  Created by splitting modules out from one big
                   file.  (process_pty.c)
*/

#include <udpinc.h>



/***************************************************************
*                                                              *
* DO_KILL                                                      *
*                                                              *
*   This routine will attempt to cleanly stop rteld processes. *
*                                                              *
*   It will attempt to stop the rtelpd process associated      *
*   associated with the specified named pipe.  If the device   *
*   name is "all" then all running rtelpd processes will be    *
*   stopped.  Note that this command requires that the user    *
*   have the appropriate privileges to stop the specified      *
*   daemon programs.                                           *
*                                                              *
***************************************************************/

do_kill(users)
udp_data *users;
{
    FILE *psfd;
    char ps_buf[256], found_pipe[80];
    int found_pid;

    dbgout("In do_kill");

    /* General theory here is that we first search for a ps line that
       has the string "rtel" in it.  Next extract the PID and pipe
       name.  If we are killing all rtel processes, or the named pipe
       the user specified matches the one found, kill the process.  */
    
    psfd = popen(PS_CMD, "r");
    while(fgets(ps_buf, sizeof(ps_buf), psfd)){
	if(strstr(ps_buf, "rtel")){
	    if(ps_line_parse(ps_buf, found_pipe, &found_pid)){
		if((!strcmp(users->pipe, "all")) || (!strcmp(users->pipe, found_pipe))){
		    printf("Attempting to kill RTEL process, PID -- %d, named pipe \"%s\"\r\n",
			   found_pid, found_pipe);
    		    sprintf(ps_buf, "kill -9 %d", found_pid);
		    system(ps_buf);
		}
	    }
	}
    }
    pclose(psfd);
    return(OK);
}




/***************************************************************
*                                                              *
* DO_REVIEW                                                    *
*                                                              *
*   This routine will show what rtelpd processes are currently *
*   running.  To do this, it actually issues a ps command and  *
*   examines all processes running on the system.  When an     *
*   RTEL process is found, it parses the command line to get   *
*   the pipe name and then looks in the server_hosts file to   *
*   get the information about the network connection.          *
*                                                              *
***************************************************************/

do_review(users)
udp_data *users;
{
    FILE *psfd;
    char found_pipe[80], ps_buf[256];
    int found_pid;

    dbgout("In do_review");

    printf("\r\n");
    printf("    Pipe             Service               Host              PID\r\n");
    printf("--------------    ----------------    ----------------    ----------\r\n");
	
    /* General theory here is that we first search for a ps line that
       has the string "rtel" in it.  Then extract the PID and pipe
       name, and use the pipe name to lookup the entry in the server_hosts
       file.  This will get us the server and service information to
       actually print for the requesting user.  */
    
    psfd = popen(PS_CMD, "r");
    while(fgets(ps_buf, sizeof(ps_buf), psfd)){
	if(strstr(ps_buf, "rtel")){
	    if(ps_line_parse(ps_buf, found_pipe, &found_pid)){
		find_dev_name(found_pipe, users);
		if(users->svcnm[0]){
		    printf("%-14s    %-16s    %-16s    %d\r\n",
			   found_pipe, users->svcnm, users->host, found_pid);
		}
		else{
		    printf("%-14s    Port Number %-4s    %-16s    %d\r\n",
			   found_pipe, users->hport, users->host, found_pid);
		}
	    }
	}
    }
    printf("\r\n");
    pclose(psfd);
    return(OK);
}



/***************************************************************
*                                                              *
* DO_SETUP                                                     *
*                                                              *
*   This routine loop through the server_hosts file looking    *
*   for lines with a "/" in the first column.  It will then    *
*   create a rtelpd command to start each daemon process and   *
*   use the system routine to fork off the appropriate daemon. *
*                                                              *
***************************************************************/

do_setup(users)
udp_data *users;
{
    FILE *filep;
    char opts[80], buf[256];
    int rval;

    dbgout("In do_setup");
    
    if ((filep = fopen(hostfile, "r")) == NULL) {
	printf ("Unable to open server_hosts file %s, errno--> %d\r\n", hostfile, errno);
	return(PROC_ERR);
    }
    
    while(fgets(buf, sizeof(buf), filep) != NULL) {
	if((buf[0] != '#') && (buf[0] == '/')) {
	    if ((rval = sscanf (buf, "%s %s %s %s %[ -z]", 
				users->pipe, users->host, users->svcnm,
				users->passwd, opts)) < 5) {
		printf("Not enough parameters on line:\r\n%s\r\n", buf);
		continue;
	    }
	    
	    /* Check to see if a rtelpd process already exists for this
	       pipe.  If so, do not start up another one.  */
	    
	    if(rtelpd_exists(users, users->pipe)){
		printf("RTELPD already running for named pipe \"%s\"\r\n", users->pipe);
	    }
	    else {
		printf("Starting RTELPD daemon for named pipe \"%s\"\r\n", users->pipe);
		sprintf(buf,"%s/rtelpd -n%s ", rtel_dir, users->pipe);
		system(buf);
	    }
	}
    }
    return(OK);
}



/***************************************************************
*                                                              *
* DO_TALK                                                      *
*                                                              *
*   This routine is used to actually talk to the server.       *
*                                                              *
***************************************************************/

do_talk(users)
udp_data *users;
{
    FILE *psfd;
    char     *arglist[10], args[10][40];
    char     found_pipe[80], ps_buf[256], *ptr1, *ptr2;
    unsigned short euid;
    int      argc, fd, found_pid, optlen, pipe_fd, stat;
    
    dbgout("In do_talk");
    
    /* If the user did not specify a service name or host name, look up
       the entry in the server_hosts file to try to find a device entry
       with the associated information.
       
       If the device entry is found, construct a bogus argc/argv structure
       to pass to rteld_parse.  This is done to handle the arguments that
       the user may have put into the options field of the server_hosts file.
       Note that arguments may be specified both on the command line and in
       the server_hosts file.  If the options field in the server_hosts file
       contains "none", simply bypass this whole exercise.  */
    
    if(!users->svcnm[0] && !users->host[0]){
	if((stat = find_dev_name(users->pipe, users)) < 0) {
	    printf("Lookup failure in for %s in server_hosts file\r\n", users->pipe);
	    return(PROC_ERR);
	}
	
	if(strcmp(users->option, "none")){
	    argc = 1;
	    arglist[0] = args[0];
	    strcpy(args[0], "bogus");
	    
	    ptr1 = users->option;
	    optlen = strlen(users->option);
	    
	    ptr2 = ptr1 + optlen;
	    while(*--ptr2 == ' '){
		optlen--;
	    }
	    
	    dprintf2(dbgbuf, "Processing options field %s, length %d",
		     users->option, optlen);
	    dbgout(dbgbuf);
	    
	    while(ptr1 < (users->option + optlen)){
		ptr2 = ptr1;
		while(*ptr2 && (*ptr2 != ' '))ptr2++;
		*ptr2++ = 0;
		arglist[argc] = args[argc];
		strcpy(arglist[argc++], ptr1);
		ptr1 = ptr2;
	    }
	    
	    if((stat = rteld_parse(argc, arglist, users)) == HELP){
		return(PROC_ERR);
	    }
	}
    }

    /* From this point on, we will be running as either a system daemon
       process or as a child process of the original users's controlling
       terminal.  If running as a system daemon, call become_daemon.
       If running as a child process, fork to return control to the user's
       terminal, close all open file descriptors and start handling the
       data stream.  */
    
    if (users->flags & BE_DAEMON) {
	if((euid = geteuid()) != 0){
	    printf ("Must be root to create a daemon process!!\r\n");
	    return(PROC_ERR);
	}
	if (become_daemon(users) < 0){
	    return(PROC_ERR);
	}
	dbgout("Running as daemon process");
    }
    else {
	if (fork()){
	    exit (0);
	}
	for (fd=0; fd < 10; fd++) close (fd);
	dbgout("Running as disassociated child process");
    }
    
    /* Open the pipe and then process the data. The pipe open will block 
       until there is data to read, then return.  Process_pipe will move
       the data from the pipe to the net.  */
    
    printf("Starting RTELPD process for pipe \"%s\"\r\n", users->pipe);
    if ((stat = open_pipe(users, &pipe_fd)) < 0){
        return(stat);
    }
    
    if ((stat = process_pipe(users, pipe_fd)) < 0){
	return(stat);
    }
}



/***************************************************************
*                                                              *
* RTELPD_EXISTS                                                *
*                                                              *
*   This routine will determine if an RTELPD process for the   *
*   specified pipe name exists.                                *
*                                                              *
***************************************************************/

int rtelpd_exists(users, pipe_name)
udp_data *users;
char *pipe_name;
{
    FILE *psfd;
    char found_pipe[80], ps_buf[256];
    int found_pid;

    psfd = popen(PS_CMD, "r");
    while(fgets(ps_buf, sizeof(ps_buf), psfd)){
	if(strstr(ps_buf, "rtel")){
	    if(ps_line_parse(ps_buf, found_pipe, &found_pid)){
		if(!strcmp(users->pipe, found_pipe)){
		    return(TRUE);
		}
	    }
	}
    }
    pclose(psfd);
    return(FALSE);
}
