/***************************************************************\
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: errors                                         *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This routine will log informational or fatal error   *
*         messages.                                            *
*                                                              *
*       Input:                                                 *
*         The reject field in the udp_data packet must contain *
*         the error reason.  Any other information required by *
*         the error message must be valid in the udp_data      *
*         structure.                                           *
*                                                              *
*         The destination of the message is determined by the  *
*         whichfile parameter.  There are four possible        *
*         destinations:                                        *
*                                                              *
*            DAEMON   -- Log message to daemon error file      *
*            RTEL     -- Log message to spooling error file    *
*            CONSOLE  -- Log message to machine's console      *
*            TERMINAL -- Log messages to user's terminal       *
*                                                              *
*       Output:                                                *
*         The message is logged.                               *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL 16-jun-92  Removed unneeded error codes.

   KCS 29-oct-91  Added new message, rtel_qfull.

   KCS 07-may-91  Added message field to errs() call, and err_handler.

   RKL 09-jan-91  Changed RTAP to RTEL.
   
   */

#include <udpinc.h>

int errs (users, errnumb, mssg)
udp_data *users;
int errnumb;
char *mssg;
{
    int i;

    /* 	If we have used up all the error message buffers, toss the oldest
	one, as presumably the latest ones are the most important.  */
    
    if(users->next == MAX_MSGS){
	for(i=1; i < MAX_MSGS; i++){
	    strcpy(users->ermssg[i - 1], users->ermssg[i]);
	    users->erarray[i - 1] = users->erarray[i];
	}
	users->next = MAX_MSGS - 1;
    }

    strcpy((users->ermssg[users->next]), mssg);
    users->erarray[users->next] = errnumb;
    users->next++;
    return OK;
}



int wrerr(users, whichfile)
udp_data *users;
int whichfile;
{
    char output[132];
    int i, firstime=0;
    
    for (i=0; i<users->next; i++) {
	switch (users->erarray[i]) {
	    case 0:{
		break;
	    }
	    
	    
	    /******************************************************
	     *                                                     *
	     *  UNIX reject/disconnect codes                       *
	     *                                                     *
	     ******************************************************/
	    
	    
	    case ENOENT:{
		sprintf(output,"%%ENOENT:  No such file or directory");
		break;
	    }
	    case EIO:{
		sprintf(output,"%%EIO: I/O error");
		break;
	    }
	    case ENXIO:{
		sprintf(output,"%%ENXIO: No such device or address");
		break;
	    }
	    case EBADF:{
		sprintf(output,"%%EBADF: Bad file number");
		break;
	    }
	    case ENOMEM:{
		sprintf(output,"%%ENOMEM: Not enough core");
		break;
	    }
	    case EACCES:{
		sprintf(output,"%%EACCES: Permission denied. Link or privs violation");
		break;
	    }
	    case EFAULT:{
		sprintf(output,"%%EFAULT: Bad address");
		break;
	    }
	    case EEXIST:{
		sprintf(output,"%%EEXIST: File already exists");
		break;
	    }
	    case ENODEV:{
		sprintf(output,"%%ENODEV: No such device");
		break;
	    }
	    case ENOTDIR:{
		sprintf(output,"%%ENOTDIR: Not a directory");
		break;
	    }
	    case EINVAL:{
		sprintf(output,"%%EINVAL: Invalid argument");
		break;
	    }
	    case EROFS:{
		sprintf(output,"%%EROFS: Read only file system");
		break;
	    }
	    case EPIPE:{
		sprintf(output,"%%EPIPE: Pipe broken during print");
		break;
	    }
	    case EWOULDBLOCK:{
		sprintf(output,"%%EWOULDBLK: Operation would block");
		break;
	    }
	    case ENOTSOCK:{
		sprintf(output,"%%ENOTSOCK: Socket operation on non-socket");
		break;
	    }
	    case EDESTADDRREQ:{
		sprintf(output,"%%EDESTADDRREQ: Destination address required");
		break;
	    }
	    case EPROTONOSUPPORT:{
		sprintf(output,"%%EPROTONOSUPPORT: Protocol not supported");
		break;
	    }
	    case ENETDOWN:{
		sprintf(output,"%%ENETDOWN: Network is down");
		break;
	    }
	    case ENETUNREACH:{
		sprintf(output,"%%ENETUNREACH: Network is unreachable");
		break;
	    }
	    case ENETRESET:{
		sprintf(output,"%%ENETRESET: Network dropped connection on reset");
		break;
	    }
	    case ECONNABORTED:{
		sprintf(output,"%%ECONNABORTED: Network connection aborted");
		break;
	    }
	    case ECONNRESET:{
		sprintf(output,"%%ECONNRESET: Connection reset by peer");
		break;
	    }
	    case ENOBUFS:{
		sprintf(output,"%%ENOBUFS: No buffer space available");
		break;
	    }
	    case EISCONN:{
		sprintf(output,"%%EISCONN: Socket is already connected");
		break;
	    }
	    case ENOTCONN:{
		sprintf(output,"%%ENOTCONN: Socket is not connected");
		break;
	    }
	    case ESHUTDOWN:{
		sprintf(output,"%%ESHUTDOWN: Can't send after socket shutdown");
		break;
	    }
	    case ETIMEDOUT:{
		sprintf(output,"%%ETIMEDOUT: Connection timed out");
		break;
	    }
	    case ECONNREFUSED:{
		sprintf(output,"%%ECONNREFUSED: Connection refused");
		break;
	    }
	    case EHOSTDOWN:{
		sprintf(output,"%%EHOSTDOWN: Host is down");
		break;
	    }
	    
	    
	    /******************************************************
	     *                                                     *
	     *  Terminal server reject/disconnect codes            *
	     *                                                     *
	     ******************************************************/
	    
	    
	    case LAT_REJNORMAL:{
		sprintf(output,"%%LAT_REJNORMAL: Normal node close");
		break;
	    }
	    case LAT_REJCIROOR:{
		sprintf(output,"%%LAT_REJCIROOR: Circuit timer out of range");
		break;
	    }
	    case LAT_REJSHUTDOWN:{
		sprintf(output,"%%LAT_REJSHUTDOWN: System shutting down");
		break;
	    }
	    case LAT_REJCOMPROTO:{
		sprintf(output,"%%LAT_REJCOMPROTO: Communication protocol error");
		break;
	    }
	    case LAT_REJINVSVC:{
		sprintf(output,"%%LAT_REJINVSVC: Invalid service class");
		break;
	    }
	    case LAT_REJINSRES:{
		sprintf(output,"%%LAT_REJINSRES: Insufficient node resources");
		break;
	    }
	    case LAT_REJINUSE:{
		sprintf(output,"%%LAT_REJINUSE: Service in use");
		break;
	    }
	    case LAT_REJNOSUCHSV:{
		sprintf(output,"%%LAT_REJNOSUCHSV: No such service");
		break;
	    }
	    case LAT_REJSVDIS:{
		sprintf(output,"%%LAT_REJSVDIS: Service disabled");
		break;
	    }
	    case LAT_REJSVPORT:{
		sprintf(output,"%%LAT_REJSVPORT: Service not on requested port");
		break;
	    }
	    case LAT_REJPORT:{
		sprintf(output,"%%LAT_REJPORT: Object port not known");
		break;
	    }
	    case LAT_REJINVPASS:{
		sprintf(output,"%%LAT_REJINVPASS: Invalid password");
		break;
	    }
	    case LAT_REJQUEENT:{
		sprintf(output,"%%LAT_REJQUEENT: Queue request not found");
		break;
	    }
	    case LAT_REJIMMACC:{
		sprintf(output,"%%LAT_REJIMACC: Immediate access denied");
		break;
	    }
	    case LAT_REJACC:{
		sprintf(output,"%%LAT_REJACC: Access denied");
		break;
	    }
	    case LAT_REJCORSOL:{
		sprintf(output,"%%LAT_REJCORSOL: Corrupt solicit");
		break;
	    }
	    
	    
	    /******************************************************
	     *                                                     *
	     *  RTEL print-driver specific errors                  *
	     *                                                     *
	     ******************************************************/
	    
	    case RTEL_NOCONN:{
		if(strlen(users->svcnm)){
		    sprintf(output,"%%RTEL_NOCONN: Couldn't form connection to %s",users->svcnm);
		}
		else{
		    sprintf(output,"%%RTEL_NOCONN: Couldn't form connection to %s",users->hport);
		}
		break;
	    }
	    case RTEL_BADSOCK:{
		sprintf(output,"%%RTEL_BADSOCK: Got a bad socket from getsockname()");
		break;
	    }
	    case RTEL_FINNED:{
		sprintf(output,"%%RTEL_FINNED: Network read failed to server %s",users->host);
		break;
	    }
	    case RTEL_BADDATA:{
		sprintf(output,"%%RTEL_BADDATA: Got invalid data from server %s",
			users->host);
		break;
	    }
	    case RTEL_NODAMN:{
		sprintf(output,"%%RTEL_NODAMN: UDP daemon files are absent");
		break;
	    }
	    case RTEL_BADRD:{
		sprintf(output,"%%RTEL_BADRD: Error reading port/pid from UDP-Daemon");
		break;
	    }
	    case RTEL_HSTNTFND:{
		sprintf(output,"%%RTEL_HSTNTFND: Host '%s' not found in /etc/hosts",users->host);
		break;
	    }
	    case RTEL_Q_HSTNTFND:{
		sprintf(output,"%%RTEL_HSTNTFND: '%s' not found in server_hosts",users->host);
		break;
	    }
	    case RTEL_BADOPN:{
		sprintf(output,"%%RTEL_BADOPN: Couldn't open file %s", users->fname);
		break;
	    }
	    case RTEL_TIMSTMP:{
		sprintf(output,"%%DM_TIMSTMP: Error writing timestamp");
		break;
	    }
	    case RTEL_NOFILE:{
		sprintf(output,"%%RTEL_NOFILE: Filename missing with print-request");
		break;
	    }
	    case RTEL_NOTFINISHED:{
		sprintf(output,"%%RTEL_NOTFINISHED: Print job exited with errors");
		break;
	    }
	    case RTEL_BUSY:{
		sprintf(output,"%%ERROR: Requested service is busy");
		break;
	    }
	    case RTEL_CNTOPNJOB:{
		sprintf(output, "%%ERROR: Could not open %s file", jobstat);
		break;
	    }
	    case RTEL_BADSCANF:{
		sprintf(output,"%%ERROR: Parameters missing in server_hosts entry");
		break;
	    }
	    case RTEL_QFULL: {
	        sprintf(output,"%%RTEL_QFULL: Job %s to %s, waiting for queue-reduction",
			users->fname, users->host);
		break;
	    }
	    case RTEL_NOBANNER: {
	        sprintf(output,"%%RTEL_NOBANNER: Could not open banner file %s",users->fname);
		break;
	    }
	    case RTEL_JOBHUNG: {
	        sprintf(output,"%%RTEL_JOBHUNG: Job stalled and being aborted");
		break;
	    }
	    case RTEL_ERRSTR: {
		sprintf(output, "   Error string -- |%s|", users->fname);
		break;
	    }
	    default:{
	        sprintf(output,"%%ERROR: Error encountered with no match, errno: %d",
			errno);
		break;
	    }
	}

	/* Now move the error to the appropriate logging location. */
	
	if (whichfile == RTEL) {
	    if (!firstime++) {
		timestamp(jobstat,users->fname);
		err_handler(jobstat,"%%ERROR:  The following error(s) have occurred:", "");
	    }
	    err_handler(jobstat, output, users->ermssg[i]);
	}
	else if (whichfile == CONSOLE) {
	    wrcons(output);
	    wrcons(users->ermssg[i]);
	    wrcons("\n");
	}
	else if (whichfile == TERMINAL) {
	    printf("%s\n", output);
	    printf("%s\n", users->ermssg[i]);
	}
    }
    return(OK);
}
