/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Richard K. Lyman                               *
*       Module: kill_files.c                                   *
*       Created: Wed Jan 22  18:34:00 1992                     *
*       Abstract:                                              *
*         These routines will handle kill file creation,       *
*         checking and deletion.                               *
*                                                              *
*       Input:                                                 *
*         ??                                                   *
*                                                              *
*       Output:                                                *
*         ??                                                   *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  06-may-93  Fixed up to work with pipes rather than
                   pseudo-ttys.

   RKL  19-nov-92  Changed name server to trmsvr to fix a
                   conflict with some system include files.

   RKL  12-may-92  Use kill -1 rather than kill -HUP since some
                   machines don't seem to allow kill -HUP in
		   the bourne shell.

   RKL  12-feb-92  Changed format of kill file to only delete
                   file when executed if successful in killing
		   the rteld process.

   RKL  22-jan-92  Split away into their own module.

 */

#include <udpinc.h>


parse_kill_file(users, pid, service, trmsvr, pipename) 
udp_data *users;
int  *pid;
char *service;
char *trmsvr;
char *pipename; 
{
    FILE *kfile;
    char junkbuf[132];

    dbgout("In parse_kill_file");

    if ((kfile = fopen(users->fname, "r")) != NULL){
	fgets(junkbuf, sizeof(junkbuf), kfile);
	fscanf(kfile, "# service %s on host %s using pipe %s \n", service, trmsvr, pipename);
	fscanf(kfile, "if(kill -1 %d)\n", pid);
	fclose(kfile);

	dprintf(dbgbuf, "Processing kill file  %s", users->fname);
	dbgout(dbgbuf);
	dprintf(dbgbuf, "     service --> %s", service);
	dbgout(dbgbuf);
	dprintf(dbgbuf, "     trmsvr ---> %s", trmsvr);
	dbgout(dbgbuf);
	dprintf(dbgbuf, "     pipename -> %s", pipename);
	dbgout(dbgbuf);
	dprintf(dbgbuf, "     pid ------> %i", *pid);
	dbgout(dbgbuf);

	return(OK);
    }
    return(PROC_ERR);
}


/*  This function is called to create a file that contains the
  information needed to kill off a rouge rteld process.
  
  If the kill file already exists, a message is displayed on
  the users terminal window so that he can either terminate
  the zombie process, or complete the proper cleanup procedure
  manually by executing rteld with the -k option.
  
  It the parameter check is specified, this routine will try
  to open the kill file.  If the open succeeds, the file will
  be closed and the routine will return a failure status.  */


make_kill_file(users, check)
udp_data *users;
int check;
{
    FILE *fkill;
    char *tmp;
    int fd;
    
    /* Create the file name and try to open the kill file.  If 
       the open succeeds, return an error status since we should
       not be able to find the file at this point. */
    
    tmp = (char *) strrchr(users->pipe,'/'); 
    tmp++; 
    sprintf(users->fname, "%s/%s", rtel_files, tmp);
    dprintf(dbgbuf, "Attempting to create kill file %s", users->fname);
    dbgout(dbgbuf);

    if((fd = open(users->fname, O_RDONLY)) >= 0){
	errs(users, RTEL_FILEXISTS, "kill_file:make_kill_file");
	close(fd);
	return(PROC_ERR);
    }

    /* If we are just checking to see if the file exists, return
       a good status, otherwise create the file and fill it with:

           #!/bin/sh
	   # service service_name on host host_name using pipe pipe_name 
	   if(kill -1 pid)
           then
	       rm /usr/spool/rtel/files/pipe_name
           fi
    
       This will kill with catchable signals the rtelpd process
       and remove all traces of the kill file. */

    if(check == CHECK_KILL){
	return(OK);
    }
    
    if((fd = creat(users->fname, KILL_MASK)) < 0){
	errs(users, RTEL_BADKILIO, "kill_file:kilfilcrt");
	errs(users, errno, "process_pty:kilfilcrt");
	return(PROC_ERR);
    }
    
    fkill = fdopen(fd, "w");
    fprintf(fkill, "#!/bin/sh\n");
    if(strlen(users->svcnm)){
	fprintf(fkill, "# service %s on host %s using pipe %s \n", users->svcnm, users->host, users->pipe); 
    }
    else{
	fprintf(fkill, "# service %s on host %s using pipe %s \n", users->hport, users->host, users->pipe); 
    }	
    fprintf(fkill, "if(kill -1 %d)\n", getpid());
    fprintf(fkill, "then\n");
    fprintf(fkill, "    rm %s\n", users->fname);
    fprintf(fkill, "fi\n");
    fclose(fkill);
    close(fd);
    return 0;
}
