/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: lp_filter                                      *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This is the main controlling program for the LP type *
*         spooling systems.  It will coordinate all activity   *
*         required to send an input file to a printer service  *
*         on an ETS terminal server.                           *
*                                                              *
*       Input:                                                 *
*         All input for this routine is received via a command *
*         line.  The command line arguments are:               *
*                                                              *
*            argv[0] -- Name of currently running program      *
*            argv[1] -- Request name                           *
*            argv[2] -- Login name                             *
*            argv[3] -- Job title                              *
*            argv[4] -- Number of copies to print              *
*            argv[5] -- Option string                          *
*            argv[6] -- File to print                          *
*                                                              *
*         This program will not process more than one input    *
*         file.  If multiple files are spooled with a single   *
*         request, a shell will have to repeatedly invoke this *
*         program.                                             *
*                                                              *
*         The option string can specify an option of the form  *
*         service@server.  If this option is specified, it     *
*         overrides the destination specified by the queue     *
*         name.                                                *
*                                                              *
*       Output:                                                *
*         If everything works perfectly, the requested job is  *
*         spooled to the specified printer.  If something      *
*         fails, error messages will be logged to the job      *
*         status log files.                                    *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  31-jul-92  Replace bzero with rtel_zero.  Amazing how many
                   support calls such a small thing can generate.

   RKL  15-may-91  Use extract_queue_name to get queue name from
                   argv[0].

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   KCS  07-may-91  Merged aix_filter.c to here

   RKL  09-jan-91  Changed RTAP to RTEL.


 */

#include <udpinc.h>

main(argc,argv)
int argc;
unsigned char* argv[];
{
    udp_data usr;
    int stat;

    rtel_zero(&usr, sizeof(udp_data));
    usr.tcp_socket = SOCK_RTEL;

    signal_stuff();
  
    /* Create the status file only if it doesn't currently exist! */
    if (open_jobstat_fil()) {
	errs(&usr, RTEL_CNTOPNJOB, "lp_filter:main");
	wrerr(&usr,CONSOLE);
	goto DONE;
    }
    

#ifdef AIX
    if ((stat = aix_parse(argc,argv,&usr)) != OK) goto DONE;
#else    
    if ((stat = lp_parse(argc,argv,&usr)) != OK) goto DONE;
#endif
    
    if (!usr.host[0])
      if (extract_queue_name(argv[0],&usr)) goto DONE;
    
    if ((usr.fdp = fopen(usr.fname, "r")) == NULL) {
	errs(&usr, RTEL_BADOPN, "lp_filter:main");
	errs(&usr, errno, "lp_filter:badfilopn");
	goto DONE;
    }

#ifdef DEBUG
    sprintf(dbgbuf, "Processing file %s", usr.fname);
    dbgout(dbgbuf);
#endif

    if (conn_and_send(&usr) < OK) {
	errs(&usr, RTEL_NOTFINISHED, "lp_filter:main");
    }
    
    DONE:
    if (usr.next > OK){
	wrerr(&usr, RTEL);
	dbgout("Exiting from lp_filter with bad status");
	exit(-1);
    }
    else{
	dbgout("Exiting from lp_filter with good status");
	exit (0);
    }
}
