/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: lpr_filter.c                                   *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         This is the main controlling program for the LPR     *
*         type spooling systems.  It will coordinate all       *
*         activity required to send an input file to a printer *
*         service on an ETS terminal server.                   *
*                                                              *
*       Input:                                                 *
*         The data to be printed is received for this routine  *
*         via a pipe from the parent spooler process.  (It     *
*         arrives on STDIN)  The name of the currently running *
*         program will be used to determine the name of the    *
*         destination queue.                                   *
*                                                              *
*       Output:                                                *
*         If everything works perfectly, the requested job is  *
*         spooled to the specified printer.  If something      *
*         fails, an error message will be written to the queue *
*         status file.                                         *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  26-oct-92  Remove extraneous length parameter on call
                   to strcpy.  Oops...

   RKL  31-jul-92  Replace bzero with rtel_zero.  Amazing how many
                   support calls such a small thing can generate.

   RKL  15-may-91  Use extract_queue_name to get queue name from
                   argv[0].

   KCS  07-may-91  Added message field to errs() call, and err_handler.

   RKL  09-jan-91  Changed RTAP to RTEL.

 */

#include <udpinc.h>

main(argc,argv)
int argc;
unsigned char* argv[];
{
    udp_data usr;

    rtel_zero(&usr, sizeof(udp_data));
    usr.tcp_socket = SOCK_RTEL;
    
    signal_stuff();
  
    /* creates the status file only if it doesn't currently exist! */
    if (open_jobstat_fil()) {
	errs(&usr, RTEL_CNTOPNJOB, "lpr_filter:main");
	wrerr(&usr,CONSOLE);
	goto DONE;
    }
    
    if (extract_queue_name(argv[0],&usr)) goto DONE;
  
    /* Set up bogus job info if the user chooses to print a banner */

    strcpy(usr.usernam, "No username available");
    strcpy(usr.fname, "No filename available");
  
    usr.fdp = stdin;
    usr.copies = 1;  /* for lpr, only print out 1 copy */
  
    if (conn_and_send(&usr) < OK) {
	errs(&usr, RTEL_NOTFINISHED, "lpr_filter:main");
    }
    
    DONE:
    if (usr.next > OK){
	dbgout("Exiting from lp_filter with bad status");
	wrerr(&usr, RTEL);
	exit(-1);
    }
    else{
	dbgout("Exiting from lp_filter with good status");
	exit (0);
    }
}
