/***************************************************************\
*                                                              *
*                   Copyright (c) 1991 Gordian Inc.            *
*                                                              *
*       Author: Rich Lyman                                     *
*       Module: open_pipe.c                                    *
*       Created: Tue 04 May 10:17:51 1993                      *
*       Abstract:                                              *
*         This module will handle opening named pipes.         *
*                                                              *
*       Input:                                                 *
*         The pipe field will contain the pipe name to         *
*         open.  This routine assumes that the pipe has        *
*         already been created by the installation script.     *
*                                                              *
*       Output:                                                *
*         If the pipe could be opened, a fd will be returned.  *
*         If the device could not be opened an error status    *
*         will be returned.                                    *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:


*/

#include "udpinc.h" 

int open_pipe(users, pipe) 
udp_data *users;
int *pipe;
{
    /* The pipe field gets built by the parser. If the pipe
       doesn't exist issue an error message. */
    
    if (users->pipe[0] == NULL) {
	errs(users, PIPE_REQUIRED, "open_pipe:no_pipe");
	return(PROC_ERR);
    }
    
    sprintf(users->fname, "%s",  users->pipe);
    dprintf(dbgbuf, "Attempting to open pipe %s", users->fname);
    dbgout(dbgbuf);
    
    if ((*pipe = open(users->pipe, O_RDONLY)) < 0) {
	errs(users, PIPE_CANTOPEN, "open_pipe:badopen");
	return(PROC_ERR);
    }
    return(OK);
}  
