/***************************************************************
*                                                              *
*                   Copyright (c) 1990 Gordian Inc.            *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: signals                                        *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         These routines handle setting up and collecting      *
*         signals in both the SYSV and BSD environments.       *
*                                                              *
*       Input:                                                 *
*         -- none --                                           *
*                                                              *
*       Output:                                                *
*         -- none --                                           *
*                                                              *
*       Disclaimer:                                            *
*         The information in this software is subject to       *
*         change without notice and should not be construed    *
*         as a commitment by the developers of this software   *
*         package.                                             *
*                                                              *
***************************************************************/

/* Edit History:

   RKL  16-sep-93  Moved connect_timeout routine into conn_srvc.

   RKL  08-feb-93  Cleaned up a couple of type mismatches and
                   remove the unneeded kid_death routine.

   RKL  31-jul-92  Added rtel_zero and rtel_copy routines to
                   finally put to sleep the bcopy/memcpy
		   bogusitites.

   RKL  17-jun-92  Upgraded waiting routine to actually wait
                   for data coming back from the server on the
		   udp channel.
   
   RKL  07-jan-92  Cleanup and completely remove the itermval
                   stuff.

   RKL  03-jul-91  Conditionalized the use of the itimerval
                   routines since SCO does not seem to have
		   them.

   RKL  17-may-91  Added signal routines hangup and kid_death
                   for support of RTEL II.

   RKL  10-jan-91  Added routine pause_abit which gives network
                   buffered data a chance to clean out a bit.

   RKL  09-jan-91  Changed RTAP to RTEL.

*/

#include <udpinc.h>


/* This routine is used by the queueing routines to keep trying
   a connection every once in a while.  This allows for recovery
   from a server crashing and not having a queued job when it
   comes back up.  */

int waiting(users)
udp_data *users;
{
    struct timeval KevClock;
    fd_set efd, rfd;
    char buff[128];
    int i, stat;
    
    /* This select will fall through if this process gets information
       back from the server via the UDP channel.  In addition, a timer
       is set to expire every 60 seconds so that we will force a retry
       to the printer service.  This will catch the case of the server
       rebooting and losing track of all information in the queue.

       The UDP message from the server will be of the form:
	   
            1 byte  -- service name length
            n bytes -- service name
	    1 byte  -- port number low
	    1 byte  -- port number high

       This will be check to confirm that it matches the information
       we originally supplied in our request. */
    
    FD_ZERO(&efd);
    FD_ZERO(&rfd);

    FD_SET(users->udp_fd, &efd);
    FD_SET(users->udp_fd, &rfd);

    KevClock.tv_sec = 60;
    KevClock.tv_usec = 0;

    dbgout("Just before select");
    if((stat = select(getdtablesize(), &rfd, 0, &efd, &KevClock)) < OK){
	dprintf(dbgbuf, "Select failed, errno: %d", errno);
	dbgout(dbgbuf);
	errs(users, errno, "waiting:badselect");
	return(PROC_ERR);
    }
    
    if(FD_ISSET(users->udp_fd, &efd)){
	dbgout("Select exception FD bit set");
	errs(users, RTEL_SELEXCEPT, "waiting:selexcept");
	return(PROC_ERR);
    }

    if(FD_ISSET(users->udp_fd, &rfd)){
	dbgout("Select read FD bit set");
	if ((stat = read(users->udp_fd, buff, sizeof(buff))) < OK){
	    dprintf(dbgbuf, "Read failed, errno: %d", errno);
	    dbgout(dbgbuf);
	    errs(users, errno, "waiting:netread");
	    return(PROC_ERR);
	}
	else {
#ifdef DEBUG
	    dbgout("UDP queued response contents: ");
	    for(i=0; i < stat; i++){
		sprintf(&(dbgbuf[i*3]), "%2.2X ", (unsigned char) buff[i]);
	    }
	    dbgout(dbgbuf);
#endif
	    return(OK);
	}
    }
    return(OK);
}

/* This routine is used by the various xx_filter programs to ensure that
   any extraneous signals are ignored.  */

int signal_stuff ()
{
    
#ifdef SIGALRM
    signal (SIGALRM, SIG_IGN);
#endif
    
#ifdef SIGUSR1
    signal (SIGUSR1, SIG_IGN);
#endif
    
#ifdef SIGUSR2
    signal (SIGUSR2, SIG_IGN);
#endif
    return(OK);
} 


/* These two routines are here simply to avoid the endless problems
   associated with trying to find bzero, bcopy and memcpy on various
   architectures.  Hard to believe that this is a problem, but it is.

   Note that the copy routine assumes that the buffers do not
   overlap. */

int rtel_copy(frombuf, tobuf, count)
char *frombuf;
char *tobuf;
int  count;
{
    while(count--) *tobuf++ = *frombuf++;
    return(OK);
}


int rtel_zero(buf, count)
char *buf;
int count;
{
    while(count--) *buf++ = 0;
    return(OK);
}
