/***************************************************************
*                                                              *
*       Copyright (c) 1990 - 1996 Gordian Inc.                 *
*                                                              *
*       Author: Kevin C. Stephan                               *
*       Module: udpinc.h                                       *
*       Created: Fri Jan  5 10:17:51 1990                      *
*       Abstract:                                              *
*         Include file for the reverse telnet host software.   *
*                                                              *
*       Modification History:                                  *
*         09-nov-94  RKL  Made file descriptors ints.          *
*                                                              *
*         13-may-94  RKL  Cleaned up and removed constants for *
*                         kill file processing.                *
*                                                              *
*         28-jan-94  RKL  Added includes for IBM backend       *
*                         and error constants.                 *
*                                                              *
*         27-jan-94  RKL  Add constants for no progress and    *
*                         abort job counters.                  *
*                                                              *
*         15-dec-93  RKL  Cleanup and remove excess garbage    *
*                                                              *
*         02-mar-93  TJC  Made changes for TLI code.           *
*                                                              *
*         24-nov-92  RKL  Added ability to log debug output    *
*                         to a file.                           *
*                                                              *
*         19-nov-92  RKL  Added another include for AT&T       *
*                         Wollengong include files.            *
*                                                              *
*         19-nov-92  RKL  Added option for form feed.          *
*                                                              *
*         10-oct-91  RKL  Corrected directories for Convergent *
*                         include files.                       *
*                                                              *
*         26-jul-91  RKL  Added changes for DYNIX systems.     *
*                                                              *
*         18-jul-91  RKL  Added changes for Stardent and ISC   *
*                         targets.                             *
*                                                              *
*         14-may-91  RKL  Merged changes for Phase II RTEL     *
*                                                              *
*         28-feb-91  RKL  Changed character declarations from  *
*                         unsigned char to char in udp data    *
*                         structure.                           *
*                                                              *
***************************************************************/

#define VERSION "V4.2/2"

/* Global include files. */

#include <sys/types.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <termio.h> 


#include <sys/file.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <netinet/in.h> 

/* OS specific include files. */

#ifdef AIX
/* #include <stropts.h> */ 
#include <sys/stream.h>
#include <sys/poll.h>
#include <sys/select.h>
#include <IN/standard.h>
#include <IN/backend.h>
#endif

#ifdef ATT_WNG
#include <sys/in.h>
#include <sys/inet.h>
#endif

#ifdef CONV
#include <sys/in.h>
#endif

#ifdef DYNIX
#define _IOCTL_
#define SIGCLD SIGCHLD
FILE *popen();
#endif

#ifdef ISC
#include <net/errno.h>
#endif

#ifdef ST3
include <sys/ioctl.h>
#endif

#ifdef SVR4
#include <unistd.h>
#define getdtablesize()  ((int) sysconf(_SC_OPEN_MAX))
#endif
 
#ifdef TLI  
#include <tiuser.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <poll.h> 
#endif

/* This is a bug in SCO's TCP/IP software.  It may be fixed some day... */

#ifdef SCO
#define getdtablesize() 60
#endif

/************************ connection defines ***********************/
/* these values must not change since they are used by the server. */

#define QD              0x40
#define RECONNECT       0x80

#define QUEUED          1
#define RTEL_PROT       2

#define TYPE_END        0
#define TYPE_VERS       1
#define TYPE_CONN       2
#define TYPE_REPL       3
#define TYPE_SNMP_PORT  3
#define TYPE_SRCHSTNAM  4
#define TYPE_DSTHSTNAM  5
#define TYPE_DST_PORT   6

#define TCP             6
#define UDP             17

#define WILL            0xfb
#define WONT            0xfc

/**********  End of server (non-changable) parameters. ***********/

/* Major rteld functions */
#define SETUP         1
#define TALK          2
#define KILL          3
#define REVIEW        4
#define HELP          5
#define VERS          6

/* Miscellaneous defines */
#define BEENQUEUED    1
#define RETRY_CONN    2

#define FALSE         0
#define LING_TIM      180
#define OK            0
#define PROC_ERR     -1
#define TRUE          1

/* These constants specify when to inform the user that a job may be
   stuck and when to abort the job if not progress has been made.  They
   are both in seconds.  */

#define NO_PROGRESS   60
#define ABORT_JOB     600

/* Socket number to use for RTEL connections */
#define SOCK_RTEL     2000

/* Flag definitions (xxx->flags) */
#define OPT_BANNER    0x01
#define OPT_BINARY    0x02
#define OPT_EXPTAB    0x04
#define OPT_POST      0x08
#define OPT_FORM      0x10
#define BE_DAEMON     0x20
#define LFCR_PIPE     0x40
#define RETRY         0x1000
#define ALLOW_QUE     0x2000
#define CONNECTED     0x4000

/********************** buffer and file defines **********************/

#ifdef BSD
#define PS_CMD "ps -auxww"
#else
#define PS_CMD "ps -ef"
#endif

#define BUFFSIZE   512
#define LOGFLSIZ 20000
#define MAXLEN     132
#define MAX_MSGS    10

#define CR         "\r"
#define LF         "\n"
#define CRLF       "\r\n"

#define RTEL      1      /* These constants define logfile locations. */
#define CONSOLE   2
#define TERMINAL  3

/**********************   Reject codes from server  **********************/
#define LAT_BASE        10000           /* when referencing, subtract base */
#define LAT_REJNORMAL   LAT_BASE+0      /* normal node close */
#define LAT_REJCIROOR   LAT_BASE+1      /* circuit timer out of range */
#define LAT_REJSHUTDOWN LAT_BASE+2      /* system shutting down */
#define LAT_REJCOMPROTO LAT_BASE+3      /* communication protocol error */
#define LAT_REJINVSVC   LAT_BASE+4      /* invalid service class */
#define LAT_REJINSRES   LAT_BASE+5      /* insufficient node resources */
#define LAT_REJINUSE    LAT_BASE+6      /* service in use */
#define LAT_REJNOSUCHSV LAT_BASE+7      /* no such service */
#define LAT_REJSVDIS    LAT_BASE+8      /* service disabled */
#define LAT_REJSVPORT   LAT_BASE+9      /* service not on requested port */
#define LAT_REJPORT     LAT_BASE+10     /* object port not known */
#define LAT_REJINVPASS  LAT_BASE+11     /* invalid password */
#define LAT_REJQUEENT   LAT_BASE+12     /* queue request not in found */
#define LAT_REJIMMACC   LAT_BASE+13     /* immediate access denied */
#define LAT_REJACC      LAT_BASE+14     /* access denied */
#define LAT_REJCORSOL   LAT_BASE+15     /* corrupt solicit */

/**********************    RTEL processing errors   **********************/
#define RTEL_BASE        11000
#define RTEL_HSTNTFND    RTEL_BASE+1      /* host missing in /etc/host file */
#define RTEL_NOCONN      RTEL_BASE+2      /* tcp connection attempt failed */
#define RTEL_BADSOCK     RTEL_BASE+3      /* got a bad socket from net-sftwr */
#define RTEL_SELEXCEPT   RTEL_BASE+4      /* exception during select */
#define RTEL_BADWR       RTEL_BASE+5      /* bad port/pid write to queue file*/
#define RTEL_FINNED      RTEL_BASE+6      /* got finned on connect request */
#define RTEL_BADDATA     RTEL_BASE+7      /* got confusing data from tserver */
#define RTEL_NODAMN      RTEL_BASE+8      /* daemon files missing */
#define RTEL_BADRD       RTEL_BASE+9      /* bad read from daemon file */
#define RTEL_BADHSTOPN   RTEL_BASE+10     /* bad open on server_hosts file */
#define RTEL_BADOPN      RTEL_BASE+11     /* bad open on status files */
#define RTEL_TIMSTMP     RTEL_BASE+12     /* couldn't timestamp status files */
#define RTEL_NOFILE      RTEL_BASE+13     /* no filename w/print-reqiest */
#define RTEL_NOTFINISHED RTEL_BASE+14     /* job finished with errors */
#define RTEL_BUSY        RTEL_BASE+19     /* service requested is busy */
#define RTEL_CNTOPNJOB   RTEL_BASE+20     /* could not open job_status file */
#define RTEL_Q_HSTNTFND  RTEL_BASE+21     /* host missing in server_hosts */
#define RTEL_Q_DEVNTFND  RTEL_BASE+22     /* tty/dev missing in server_hosts */
#define RTEL_BADSCANF    RTEL_BASE+23     /* bad scanf in server_hosts file */
#define RTEL_ABORT       RTEL_BASE+24     /* read/write damon, child aborted */
#define RTEL_NOBANNER    RTEL_BASE+25     /* unable to open banner file */
#define RTEL_QFULL       RTEL_BASE+26     /* request denied, tserver q is full */
#define RTEL_ERRSTR      RTEL_BASE+27     /* print a generic error string */
#define RTEL_JOBHUNG     RTEL_BASE+28     /* No progress processing job. */

/**********************      Named pipe errors      **********************/
#define PIPE_BASE        16000
#define PIPE_REQUIRED    PIPE_BASE+1        /* pipe name required */
#define PIPE_CANTOPEN    PIPE_BASE+2        /* can't open pipe */
#define PIPE_POLLERR     PIPE_BASE+3        /* poll error */
#define PIPE_GETMSGERR   PIPE_BASE+4        /* getmsg error */


/**********************        Streams stuff        **********************/
extern  int              t_errno;

#ifdef SCO
#define TLI_TCP_DEV      "/dev/inet/tcp"
#define TLI_UDP_DEV      "/dev/inet/udp"
#else 
#define TLI_TCP_DEV      "/dev/tcp"
#define TLI_UDP_DEV      "/dev/udp"
#endif

#define TLI_BASE         15000
#define TLI_ALLOCERR     TLI_BASE+1       /* t_alloc error */
#define TLI_BINDERR      TLI_BASE+2       /* t_bind error */
#define TLI_BADFCNTL     TLI_BASE+3       /* fcntl error */
#define TLI_NOCONNECT    TLI_BASE+4       /* t_connect error*/
#define TLI_BADOPEN      TLI_BASE+5       /* t_open error */
#define TLI_BADPUSH      TLI_BASE+6       /* ioctl error */ 
 
/********************** RTEL file name defines  **********************/
#define rtel_dir     "/usr/spool/rtel"
#define jobstat      "/usr/spool/rtel/job_status"
#define hostfile     "/usr/spool/rtel/server_hosts"


/***************** Structures to use if DEBUG flag is defined *************/
/* Note that dbgout can be defined to go to either the system console     */
/* or to a file which will be flushed after each write.  This should      */
/* make debugging easier.   Use -DDEBUG -DDBGFILE to send output to file. */
/**************************************************************************/
#ifdef DEBUG
char dbgbuf[512];
#define dprintf(x,y,z)    sprintf(x,y,z)
#define dprintf2(x,y,z,t) sprintf(x,y,z,t)
#ifdef DBGFILE
#define dbgout(s)        wrfile(s)
#else
#define dbgout(s)        wrcons(s)
#endif
#else
#define dbgout(s)
#define dprintf(x,y,z)
#define dprintf2(x,y,z,t)
#endif

/********************** Miscellaneous Structures  **********************/
struct timerval {
  long    tv_sec;    /* seconds */
  long    tv_usec;   /* and microseconds, 10ths of seconds */
};

typedef struct{
  struct sockaddr_in
                 temp_sockdata,     /* for getting information */
                 tcp_sockdata,      /* TCP (data) session network data */
                 udp_sockdata;      /* UDP (queueing) session network data */

  FILE *         fdp;               /* file to be printed */

  unsigned char  usr_buf[BUFFSIZE]; /* buffer for read/write */

  char           ermssg[MAX_MSGS][MAXLEN], /* 10 text error messages */
                 host[MAXLEN],      /* name of target tserver */
                 svcnm[MAXLEN],     /* name of service to use */
                 passwd[MAXLEN],    /* service password */
                 hnode[MAXLEN],     /* node name for service of form svcnm@host */
                 hport[MAXLEN],     /* port number/name on the server */
                 usernam[MAXLEN],   /* name of the user who requested job */
                 fname[MAXLEN],     /* name of file to print */
                 option[MAXLEN],    /* options field specified for rteld */
                 pipe[MAXLEN];      /* pipe name from -n option */

  int            tcp_fd,            /* TCP file descriptor for data transfer */
                 udp_fd;            /* UDP file descriptor for queueing */

  unsigned int   next,              /* pointer of where to put next error */
                 copies,            /* number of copies to print */
                 flags,             /* for setting up connection */
                 erarray[MAX_MSGS]; /* errors to log */

  unsigned short tcp_socket,        /* TCP socket number for data stream */
                 uniq_id,           /* original outgoing port number for queueing */
                 udp_socket;        /* UDP socket number for queueing */

} udp_data;
